@@ -322,6 +322,7 @@ open_calendar (ECalBackendSync *backend, EDataCal *cal, gboolean only_if_exists,
 
 		d(printf ("ECBE : cal is offline .. load cache\n"));
 
+		
 		cbex->priv->read_only = TRUE;
 		source = e_cal_backend_get_source (E_CAL_BACKEND (cbex));
 		display_contents = e_source_get_property (source, "offline_sync");
@@ -340,6 +341,11 @@ open_calendar (ECalBackendSync *backend, EDataCal *cal, gboolean only_if_exists,
 				cbex->folder = exchange_account_get_folder (cbex->account, uristr);
 			}
 		}
+		
+		if (cbex->priv->is_loaded) {
+			g_mutex_unlock (cbex->priv->open_lock);
+			return GNOME_Evolution_Calendar_Success;
+		}
 
 		euri = e2k_uri_new (uristr);
 		load_status = load_cache (cbex, euri);
@@ -351,6 +357,12 @@ open_calendar (ECalBackendSync *backend, EDataCal *cal, gboolean only_if_exists,
 		return load_status;
 	}
 		
+	/* What else to check */
+	if (cbex->priv->is_loaded && cbex->account && exchange_account_get_context (cbex->account)) {
+		g_mutex_unlock (cbex->priv->open_lock);
+		return GNOME_Evolution_Calendar_Success;
+	}
+		
 	/* Make sure we have an open connection */
 	/* This steals the ExchangeAccount from ExchangeComponent */
 	cbex->account = exchange_component_get_account_for_uri (global_exchange_component, uristr);
@@ -492,8 +504,23 @@ static void
 uncache (gpointer key, gpointer value, gpointer data)
 {
 	ECalBackendExchange *cbex = data;
+	ECalBackend *backend = E_CAL_BACKEND (cbex);
+	ECalComponentId *id = g_new0 (ECalComponentId, 1);
+	ECalBackendExchangeComponent *ecomp;
 
+	ecomp = (ECalBackendExchangeComponent *)value;
+	
+	/* FIXME Need get the recurrence id here */
+	id->uid = g_strdup (key);
+	id->rid = NULL;
+	if (ecomp->icomp) {
+		/* FIXME somehow the query does not match with the component in some cases, so user needs to press a
+		   clear to get rid of the component from the view in that case*/
+		e_cal_backend_notify_object_removed (backend, id, icalcomponent_as_ical_string (ecomp->icomp)
+				, NULL);
+	}
 	g_hash_table_remove (cbex->priv->objects, key);
+	e_cal_component_free_id (id);
 }
 
 void
@@ -620,7 +647,8 @@ discard_detached_instance (ECalBackendExchangeComponent *ecomp,
 gboolean
 e_cal_backend_exchange_modify_object (ECalBackendExchange *cbex,
 				      icalcomponent *comp,
-				      CalObjModType mod)
+				      CalObjModType mod,
+				      gboolean discard_detached)
 {
 	ECalBackendExchangeComponent *ecomp;
 	const char *uid;
@@ -638,13 +666,17 @@ e_cal_backend_exchange_modify_object (ECalBackendExchange *cbex,
 	if (!ecomp)
 		return FALSE;
 
-	if (mod == CALOBJ_MOD_ALL || icaltime_is_null_time (rid)) {
+	if (mod == CALOBJ_MOD_ALL || icaltime_is_null_time (rid) || discard_detached) {
 		icalcomponent_free (ecomp->icomp);
 		ecomp->icomp = icalcomponent_new_clone (comp);
+		if (discard_detached && !icaltime_is_null_time (rid))
+			discard_detached_instance (ecomp, rid);
 	} else {
-		discard_detached_instance (ecomp, rid);
 		ecomp->instances = g_list_prepend (ecomp->instances,
 						   icalcomponent_new_clone (comp));
+		/*FIXME This is a workaround to cover the timezone issue while setting rid from evolution */
+		if (ecomp->icomp)
+			e_cal_util_remove_instances (ecomp->icomp, rid, CALOBJ_MOD_THIS);
 	}
 
 	save_cache (cbex);
@@ -1216,6 +1248,7 @@ set_mode (ECalBackend *backend, CalMode mode)
 {
 	ECalBackendExchange *cbex;
 	ECalBackendExchangePrivate *priv;
+	gboolean re_open = FALSE;
 	
 	cbex = E_CAL_BACKEND_EXCHANGE (backend);
 	priv = cbex->priv;
@@ -1228,7 +1261,10 @@ set_mode (ECalBackend *backend, CalMode mode)
 			cal_mode_to_corba (mode));
 	}
 
+
 	g_mutex_lock (priv->set_lock);	
+	if ((priv->mode == CAL_MODE_LOCAL) && (mode == CAL_MODE_REMOTE))
+		re_open = TRUE;
 
 	switch (mode) {
 	
@@ -1241,7 +1277,7 @@ set_mode (ECalBackend *backend, CalMode mode)
 			priv->mode = CAL_MODE_REMOTE;
 		
 			e_cal_backend_notify_readonly (backend, priv->read_only);
-			if (is_loaded (backend))
+			if (is_loaded (backend) && re_open)
 				e_cal_backend_notify_auth_required(backend);
 			break;
 
@@ -1371,11 +1407,13 @@ e_cal_backend_exchange_make_timestamp_rfc822 (time_t when)
 char *
 e_cal_backend_exchange_get_from_string (ECalBackendSync *backend, ECalComponent *comp)
 {
-	char *name, *addr;
+	char *name = NULL, *addr = NULL, *from_string = NULL;
 
 	e_cal_backend_exchange_get_from (backend, comp, &name, &addr);
-	
-	return g_strdup_printf ("\"%s\" <%s>", name, addr);
+	from_string = g_strdup_printf ("\"%s\" <%s>", name, addr);
+	g_free (name);
+	g_free (addr);
+	return from_string;
 }
 
 gchar *
@@ -1543,6 +1581,7 @@ get_attachment (ECalBackendExchange *cbex, const char *uid,
 				attach_file = g_strdup_printf ("%s/%s-%s", cbex->priv->local_attachment_store, uid, filename);
 				// Attach
 				attach_file_url = save_attach_file (attach_file, attach_data, stream_mem->buffer->len);
+				g_free (attach_data);
 				g_free (attach_file);
 				d(printf ("attach file name : %s\n", attach_file_url));
 				list = g_slist_append (list, attach_file_url);
@@ -1605,7 +1644,7 @@ build_msg ( ECalBackendExchange *cbex, ECalComponent *comp, const char *subject,
 	CamelContentType *type;
 	const char *uid;
 	char *buffer = NULL, *cid;
-	char *from_name, *from_email;
+	char *from_name = NULL, *from_email = NULL;
 	GSList *attach_list = NULL, *l, *new_attach_list = NULL;
 	char *fname, *file_contents = NULL, *filename, *dest_url, *mime_filename, *attach_file;
 	int len = 0;
@@ -1622,6 +1661,8 @@ build_msg ( ECalBackendExchange *cbex, ECalComponent *comp, const char *subject,
 	from = camel_internet_address_new ();
 	camel_internet_address_add (from, from_name, from_email);
 	camel_mime_message_set_from (msg, from);
+	g_free (from_name);
+	g_free (from_email);
 	camel_object_unref (from);
 
 	e_cal_component_get_uid (comp, &uid);
@@ -1674,7 +1715,6 @@ build_msg ( ECalBackendExchange *cbex, ECalComponent *comp, const char *subject,
 		camel_multipart_add_part (multipart, mime_part);
 		camel_object_unref (mime_part);
 		g_free (cid);
-		g_free (dest_url);
 	}
 	if (!new_attach_list) {
 		camel_object_unref (multipart);
@@ -1809,6 +1849,7 @@ init (ECalBackendExchange *cbex)
 
 	cbex->priv->set_lock = g_mutex_new ();
 	cbex->priv->open_lock = g_mutex_new ();
+	cbex->priv->cache_unseen = NULL;
 
 	e_cal_backend_sync_set_lock (E_CAL_BACKEND_SYNC (cbex), TRUE);
 }
